package com.jpsoft.smart.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.smart.config.OSSConfig;
import com.jpsoft.smart.modules.common.utils.CheckIdCard;
import com.jpsoft.smart.modules.common.utils.OSSUtil;
import com.jpsoft.smart.modules.common.utils.POIUtils;
import com.jpsoft.smart.modules.common.utils.PojoUtils;
import com.jpsoft.smart.modules.common.dto.Sort;
import com.jpsoft.smart.modules.common.dto.MessageResult;
import com.jpsoft.smart.modules.base.entity.PersonInfo;
import com.jpsoft.smart.modules.base.service.PersonInfoService;
import com.jpsoft.smart.modules.sys.entity.User;
import com.jpsoft.smart.modules.sys.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Sheet;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/personInfo")
@Api(description = "personInfo")
public class PersonInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private OSSConfig ossConfig;
    @Autowired
    private PersonInfoService personInfoService;
    @Autowired
    private UserService userService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<PersonInfo> create(){
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        PersonInfo personInfo = new PersonInfo();

        msgResult.setData(personInfo);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<PersonInfo> add(@RequestBody PersonInfo personInfo,@RequestAttribute String subject){
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {
//            personInfo.setId(UUID.randomUUID().toString());
            personInfo.setDelFlag(false);
            personInfo.setCreateBy(subject);
            personInfo.setCreateTime(new Date());

            int affectCount = personInfoService.insert(personInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(personInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<PersonInfo> edit(@PathVariable("id") String id){
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.get(id);

            if (personInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(personInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<PersonInfo> update(@RequestBody PersonInfo personInfo,@RequestAttribute String subject){
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {
            personInfo.setUpdateBy(subject);
            personInfo.setUpdateTime(new Date());

            int affectCount = personInfoService.update(personInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(personInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="保存信息")
    @PostMapping("save")
    public MessageResult<PersonInfo> save(@RequestBody PersonInfo personInfo,@RequestAttribute String subject){
        User user = userService.get(subject);
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;
            if(personInfo.getId() != null) {
                personInfo.setCompanyId(user.getCompanyId());
                personInfo.setDelFlag(false);
                personInfo.setCreateBy(subject);
                personInfo.setCreateTime(new Date());

                affectCount = personInfoService.insert(personInfo);
            }else{
                personInfo.setUpdateBy(subject);
                personInfo.setUpdateTime(new Date());

                affectCount = personInfoService.update(personInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(personInfo);
            }else{
                msgResult.setResult(false);
                msgResult.setMessage("数据库保存失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取详情信息")
    @GetMapping("detail/{id}")
    public MessageResult<PersonInfo> detail(@PathVariable("id") String id){
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.get(id);

            if (personInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(personInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.get(id);
            personInfo.setDelFlag(true);
            personInfo.setUpdateBy(subject);
            personInfo.setUpdateTime(new Date());

            int affectCount = personInfoService.update(personInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                PersonInfo personInfo = personInfoService.get(id);
                personInfo.setDelFlag(true);
                personInfo.setUpdateBy(subject);
                personInfo.setUpdateTime(new Date());

                affectCount += personInfoService.update(personInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "name",value = "姓名", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "idCard",value = "身份证", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "phone",value = "手机号", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "position1",value = "一级位置", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "position2",value = "二级位置", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "position3",value = "三级位置", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "position4",value = "四级位置", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "position5",value = "五级位置", required = false, paramType = "form",dataType = "String")
    })
    public MessageResult<Map> pageList(
            @RequestParam(value="name",defaultValue="") String name,
            @RequestParam(value="idCard",defaultValue="") String idCard,
            @RequestParam(value="phone",defaultValue="") String phone,
            @RequestParam(value="position1",defaultValue="") String position1,
            @RequestParam(value="position2",defaultValue="") String position2,
            @RequestParam(value="position3",defaultValue="") String position3,
            @RequestParam(value="position4",defaultValue="") String position4,
            @RequestParam(value="position5",defaultValue="") String position5,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(name)) {
            searchParams.put("name","%" + name + "%");
        }
        if (StringUtils.isNotEmpty(idCard)) {
            searchParams.put("name","%" + idCard + "%");
        }
        if (StringUtils.isNotEmpty(phone)) {
            searchParams.put("name","%" + phone + "%");
        }
        if (StringUtils.isNotEmpty(position1)) {
            searchParams.put("position1",position1);
        }
        if (StringUtils.isNotEmpty(position2)) {
            searchParams.put("position2",position2);
        }
        if (StringUtils.isNotEmpty(position3)) {
            searchParams.put("position3",position3);
        }
        if (StringUtils.isNotEmpty(position4)) {
            searchParams.put("position4",position4);
        }
        if (StringUtils.isNotEmpty(position5)) {
            searchParams.put("position5",position5);
        }

        Page<PersonInfo> page = personInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="人脸授权")
    @PostMapping("EnabledFace")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "人员编号", required = false, paramType = "form",dataType = "String")
    })
    public MessageResult<PersonInfo> EnabledFace(@RequestParam(value="id",defaultValue="") String id,@RequestAttribute String subject){
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.get(id);

            if(personInfo.getFaceEnabled()){
                personInfo.setFaceEnabled(false);
            }else{
                personInfo.setFaceEnabled(true);
            }
            personInfo.setUpdateBy(subject);
            personInfo.setUpdateTime(new Date());

            int affectCount = personInfoService.update(personInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(personInfo);
            }else{
                msgResult.setResult(false);
                msgResult.setMessage("数据库保存失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="刷卡授权")
    @PostMapping("EnabledCard")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "人员编号", required = false, paramType = "form",dataType = "String")
    })
    public MessageResult<PersonInfo> EnabledCard(@RequestParam(value="id",defaultValue="") String id,@RequestAttribute String subject){
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.get(id);

            if(personInfo.getCardEnabled()){
                personInfo.setCardEnabled(false);
            }else{
                personInfo.setCardEnabled(true);
            }
            personInfo.setUpdateBy(subject);
            personInfo.setUpdateTime(new Date());

            int affectCount = personInfoService.update(personInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(personInfo);
            }else{
                msgResult.setResult(false);
                msgResult.setMessage("数据库保存失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="手机授权")
    @PostMapping("EnabledApp")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "人员编号", required = false, paramType = "form",dataType = "String")
    })
    public MessageResult<PersonInfo> EnabledApp(@RequestParam(value="id",defaultValue="") String id,@RequestAttribute String subject){
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.get(id);

            if(personInfo.getAppEnabled()){
                personInfo.setAppEnabled(false);
            }else{
                personInfo.setAppEnabled(true);
            }
            personInfo.setUpdateBy(subject);
            personInfo.setUpdateTime(new Date());

            int affectCount = personInfoService.update(personInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(personInfo);
            }else{
                msgResult.setResult(false);
                msgResult.setMessage("数据库保存失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="访客授权")
    @PostMapping("EnabledGuest")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "人员编号", required = false, paramType = "form",dataType = "String")
    })
    public MessageResult<PersonInfo> EnabledGuest(@RequestParam(value="id",defaultValue="") String id,@RequestAttribute String subject){
        MessageResult<PersonInfo> msgResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.get(id);

            if(personInfo.getGuestEnabled()){
                personInfo.setGuestEnabled(false);
            }else{
                personInfo.setGuestEnabled(true);
            }
            personInfo.setUpdateBy(subject);
            personInfo.setUpdateTime(new Date());

            int affectCount = personInfoService.update(personInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(personInfo);
            }else{
                msgResult.setResult(false);
                msgResult.setMessage("数据库保存失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="导入人员（非企业账户需要输入企业ID）")
    @PostMapping("importXls")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "companyId",value = "企业ID", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "uploadFile",value = "上传文件", required = true,paramType="form", dataType = "__file")
    })
    public MessageResult<String> importXls(@RequestParam(value="companyId",defaultValue="") String companyId,
                                           MultipartFile uploadFile,
                                           @RequestAttribute String subject){
        User user = userService.get(subject);
        if(StringUtils.isEmpty(companyId)){
            companyId = user.getCompanyId();
        }

        MessageResult<String> msgResult = new MessageResult<>();
        PersonInfo personInfo = new PersonInfo();

        try {
            POIUtils poiUtils = new POIUtils(uploadFile.getInputStream());
            int sheetIndex = 0;
            Sheet sheet1 = poiUtils.getSheetAt(sheetIndex);

            int affectCount = 0;
            int failCount = 0;
            int validateColIndex = 7;

            for(int rowIndex=1 ; rowIndex<=sheet1.getLastRowNum(); rowIndex++){
                try {
                    String name = (String)poiUtils.getCellValue(sheetIndex,rowIndex,1);
                    String idCard = (String)poiUtils.getCellValue(sheetIndex,rowIndex,2);
                    String phone = (String)poiUtils.getCellValue(sheetIndex,rowIndex,3);
                    String position1 = (String)poiUtils.getCellValue(sheetIndex,rowIndex,4);
                    String position2 = (String)poiUtils.getCellValue(sheetIndex,rowIndex,5);
                    String position3 = (String)poiUtils.getCellValue(sheetIndex,rowIndex,6);
                    String position4 = (String)poiUtils.getCellValue(sheetIndex,rowIndex,7);
                    String position5 = (String)poiUtils.getCellValue(sheetIndex,rowIndex,8);
                    String enabledFace = (String)poiUtils.getCellValue(sheetIndex,rowIndex,9);
                    String enabledCard = (String)poiUtils.getCellValue(sheetIndex,rowIndex,10);
                    String enabledApp = (String)poiUtils.getCellValue(sheetIndex,rowIndex,11);
                    String enabledGuest = (String)poiUtils.getCellValue(sheetIndex,rowIndex,12);

                    if(StringUtils.isEmpty(name)){
                        break;
                    }

                    Map<String, Object> searchParams = new HashMap<>();
                    searchParams.put("companyId", companyId);
                    searchParams.put("idCard", idCard);
                    List<Sort> sortList = new ArrayList<>();
                    sortList.add(new Sort("id_","asc"));
                    Page<PersonInfo> page = personInfoService.pageSearch(searchParams, 1, 1,false, sortList);

                    if (page.size() > 0) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("身份证已导入!");
                        failCount++;
                        continue;
                    } else {
                        personInfo.setCompanyId(companyId);
                        personInfo.setName(name);

                        CheckIdCard cic = null;
                        if(StringUtils.isEmpty(idCard)){
                            sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("身份证不能为空!");
                            failCount++;
                            continue;
                        }
                        else{
                            idCard = idCard.trim().toUpperCase();
                        }

                        cic = new CheckIdCard(idCard);
                        if (!cic.validate()){
                            sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("身份证验证失败!");
                            failCount++;
                            continue;
                        }

                        personInfo.setIdCard(idCard);
                        personInfo.setPhone(phone);
                        personInfo.setPosition1(position1);
                        personInfo.setPosition2(position2);
                        personInfo.setPosition3(position3);
                        personInfo.setPosition4(position4);
                        personInfo.setPosition5(position5);
                        if(enabledFace.contains("开")) personInfo.setFaceEnabled(true); else personInfo.setFaceEnabled(false);
                        if(enabledCard.contains("开")) personInfo.setCardEnabled(true); else personInfo.setCardEnabled(false);
                        if(enabledApp.contains("开")) personInfo.setAppEnabled(true); else personInfo.setAppEnabled(false);
                        if(enabledGuest.contains("开")) personInfo.setGuestEnabled(true); else personInfo.setGuestEnabled(false);
                        personInfo.setDelFlag(false);
                        personInfo.setCreateBy(subject);
                        personInfo.setCreateTime(new Date());

                        if (personInfoService.insert(personInfo) > 0) {
                            affectCount++;
                        }
                    }
                }
                catch(Exception innerEx){
                    logger.error(innerEx.getMessage(),innerEx);
                }
            }

            if (failCount>0){
                //有导入失败的记录
                msgResult.setResult(false);
                msgResult.setMessage("数据成功导入" + affectCount + "条,有" + failCount + "条数据未导入成功，错误原因请查看报表。");

                //todo 只保留错误数据的sheet
                int rowIndex = 1;

                while(rowIndex<= sheet1.getLastRowNum()){
                    Cell cell = sheet1.getRow(rowIndex).getCell(validateColIndex);

                    if (cell==null || StringUtils.isEmpty(cell.getStringCellValue())){
                        sheet1.removeRow(sheet1.getRow(rowIndex));

                        if (rowIndex<sheet1.getLastRowNum()) {
                            sheet1.shiftRows(rowIndex + 1, sheet1.getLastRowNum(), -1); //删除后下面行上移，则不需要移动rowIndex
                        }
                    }
                    else {
                        rowIndex++;
                    }
                }

                //todo 将wb保存到oss
                ByteArrayOutputStream output = new ByteArrayOutputStream();
                poiUtils.getWorkbook().write(output);

                byte[] buffer = output.toByteArray();
                ByteArrayInputStream input = new ByteArrayInputStream(buffer);

                String downloadUrl = OSSUtil.upload(ossConfig,"import","error.xls",input);

                //todo 返回导入失败报表下载链接
                msgResult.setData(downloadUrl);
            }
            else{
                msgResult.setResult(true);
                msgResult.setMessage("数据成功导入" + affectCount + "条");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="导出人员")
    @PostMapping("exportXls")
    public MessageResult<String> exportXls(@RequestAttribute String subject){
        User user = userService.get(subject);

        MessageResult<String> msgResult = new MessageResult<>();

        try {

        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
